@@ -20,6 +20,12 @@ pub fn run() -> Result<(), FltkError> {
     }
 }
 
+/// Enable locks. This is called automatically in the beginning of the app initialization
+pub fn enable_locks() -> Result<(), FltkError> {
+    lock()?;
+    Ok(())
+}
+
 /// Locks the main UI thread
 /// # Errors
 /// Returns `FailedToLock` if locking is unsopported. This is fatal to the app
@@ -129,7 +135,7 @@ pub fn ready() -> bool {
 
 /// Quit the app
 pub fn quit() {
-    if let Some(loaded_font) = *LOADED_FONT {
+    if let Some(loaded_font) = LOADED_FONT {
         // Shouldn't fail
         unload_font(loaded_font).unwrap_or(());
     }
@@ -193,7 +199,8 @@ pub fn has_idle<F: FnMut() + 'static>(cb: F) -> bool {
 pub fn add_idle2(cb: fn()) {
     unsafe {
         let data: *mut raw::c_void = std::ptr::null_mut();
-        let callback: Option<unsafe extern "C" fn(arg1: *mut raw::c_void)> = Some(mem::transmute(cb));
+        let callback: Option<unsafe extern "C" fn(arg1: *mut raw::c_void)> =
+            Some(mem::transmute(cb));
         fl::Fl_add_idle(callback, data);
     }
 }
@@ -202,7 +209,8 @@ pub fn add_idle2(cb: fn()) {
 pub fn remove_idle2(cb: fn()) {
     unsafe {
         let data: *mut raw::c_void = std::ptr::null_mut();
-        let callback: Option<unsafe extern "C" fn(arg1: *mut raw::c_void)> = Some(mem::transmute(cb));
+        let callback: Option<unsafe extern "C" fn(arg1: *mut raw::c_void)> =
+            Some(mem::transmute(cb));
         fl::Fl_remove_idle(callback, data);
     }
 }
@@ -211,7 +219,8 @@ pub fn remove_idle2(cb: fn()) {
 pub fn has_idle2(cb: fn()) -> bool {
     unsafe {
         let data: *mut raw::c_void = std::ptr::null_mut();
-        let callback: Option<unsafe extern "C" fn(arg1: *mut raw::c_void)> = Some(mem::transmute(cb));
+        let callback: Option<unsafe extern "C" fn(arg1: *mut raw::c_void)> =
+            Some(mem::transmute(cb));
         fl::Fl_has_idle(callback, data) != 0
     }
 }
@@ -244,7 +253,7 @@ pub fn add_timeout<F: FnMut() + 'static>(tm: f64, cb: F) {
         let a: *mut Box<dyn FnMut()> = Box::into_raw(Box::new(Box::new(cb)));
         let data: *mut raw::c_void = a as *mut raw::c_void;
         let callback: Option<unsafe extern "C" fn(arg1: *mut raw::c_void)> = Some(shim);
-        fltk_sys::fl::Fl_add_timeout(tm, callback, data);
+        fl::Fl_add_timeout(tm, callback, data);
     }
 }
 
@@ -278,7 +287,7 @@ pub fn repeat_timeout<F: FnMut() + 'static>(tm: f64, cb: F) {
         let a: *mut Box<dyn FnMut()> = Box::into_raw(Box::new(Box::new(cb)));
         let data: *mut raw::c_void = a as *mut raw::c_void;
         let callback: Option<unsafe extern "C" fn(arg1: *mut raw::c_void)> = Some(shim);
-        fltk_sys::fl::Fl_repeat_timeout(tm, callback, data);
+        fl::Fl_repeat_timeout(tm, callback, data);
     }
 }
 
@@ -293,7 +302,7 @@ pub fn remove_timeout<F: FnMut() + 'static>(cb: F) {
         let a: *mut Box<dyn FnMut()> = Box::into_raw(Box::new(Box::new(cb)));
         let data: *mut raw::c_void = a as *mut raw::c_void;
         let callback: Option<unsafe extern "C" fn(arg1: *mut raw::c_void)> = Some(shim);
-        fltk_sys::fl::Fl_remove_timeout(callback, data);
+        fl::Fl_remove_timeout(callback, data);
     }
 }
 
@@ -308,7 +317,7 @@ pub fn has_timeout<F: FnMut() + 'static>(cb: F) -> bool {
         let a: *mut Box<dyn FnMut()> = Box::into_raw(Box::new(Box::new(cb)));
         let data: *mut raw::c_void = a as *mut raw::c_void;
         let callback: Option<unsafe extern "C" fn(arg1: *mut raw::c_void)> = Some(shim);
-        fltk_sys::fl::Fl_has_timeout(callback, data) != 0
+        fl::Fl_has_timeout(callback, data) != 0
     }
 }
 
@@ -333,8 +342,9 @@ pub fn has_timeout<F: FnMut() + 'static>(cb: F) -> bool {
 pub fn add_timeout2(tm: f64, cb: fn()) {
     unsafe {
         let data: *mut raw::c_void = std::ptr::null_mut();
-        let callback: Option<unsafe extern "C" fn(arg1: *mut raw::c_void)> = Some(mem::transmute(cb));
-        fltk_sys::fl::Fl_add_timeout(tm, callback, data);
+        let callback: Option<unsafe extern "C" fn(arg1: *mut raw::c_void)> =
+            Some(mem::transmute(cb));
+        fl::Fl_add_timeout(tm, callback, data);
     }
 }
 
@@ -361,8 +371,9 @@ pub fn add_timeout2(tm: f64, cb: fn()) {
 pub fn repeat_timeout2(tm: f64, cb: fn()) {
     unsafe {
         let data: *mut raw::c_void = std::ptr::null_mut();
-        let callback: Option<unsafe extern "C" fn(arg1: *mut raw::c_void)> = Some(mem::transmute(cb));
-        fltk_sys::fl::Fl_repeat_timeout(tm, callback, data);
+        let callback: Option<unsafe extern "C" fn(arg1: *mut raw::c_void)> =
+            Some(mem::transmute(cb));
+        fl::Fl_repeat_timeout(tm, callback, data);
     }
 }
 
@@ -370,8 +381,9 @@ pub fn repeat_timeout2(tm: f64, cb: fn()) {
 pub fn remove_timeout2(cb: fn()) {
     unsafe {
         let data: *mut raw::c_void = std::ptr::null_mut();
-        let callback: Option<unsafe extern "C" fn(arg1: *mut raw::c_void)> = Some(mem::transmute(cb));
-        fltk_sys::fl::Fl_remove_timeout(callback, data);
+        let callback: Option<unsafe extern "C" fn(arg1: *mut raw::c_void)> =
+            Some(mem::transmute(cb));
+        fl::Fl_remove_timeout(callback, data);
     }
 }
 
@@ -379,7 +391,27 @@ pub fn remove_timeout2(cb: fn()) {
 pub fn has_timeout2(cb: fn()) -> bool {
     unsafe {
         let data: *mut raw::c_void = std::ptr::null_mut();
-        let callback: Option<unsafe extern "C" fn(arg1: *mut raw::c_void)> = Some(mem::transmute(cb));
-        fltk_sys::fl::Fl_has_timeout(callback, data) != 0
+        let callback: Option<unsafe extern "C" fn(arg1: *mut raw::c_void)> =
+            Some(mem::transmute(cb));
+        fl::Fl_has_timeout(callback, data) != 0
     }
 }
+
+/// Add a system handler
+/// # Safety
+/// FLTK makes no assurances regarding handling by the system handler
+pub unsafe fn add_system_handler(
+    cb: Option<unsafe extern "C" fn(*mut raw::c_void, *mut raw::c_void) -> i32>,
+    data: *mut raw::c_void,
+) {
+    fl::Fl_add_system_handler(cb, data);
+}
+
+/// Add a system handler
+/// # Safety
+/// FLTK makes no assurances regarding handling by the system handler
+pub unsafe fn remove_system_handler(
+    cb: Option<unsafe extern "C" fn(*mut raw::c_void, *mut raw::c_void) -> i32>,
+) {
+    fl::Fl_remove_system_handler(cb);
+}
