@@ -207,17 +207,26 @@ pub fn draw_focus_rect(x: i32, y: i32, w: i32, h: i32) {
 
 /// Sets the drawing color
 pub fn set_draw_hex_color(color: u32) {
+    unsafe {
+        crate::app::open_display();
+    }
     let (r, g, b) = crate::utils::hex2rgb(color);
     unsafe { Fl_set_color_rgb(r, g, b) }
 }
 
 /// Sets the drawing color
 pub fn set_draw_rgb_color(r: u8, g: u8, b: u8) {
+    unsafe {
+        crate::app::open_display();
+    }
     unsafe { Fl_set_color_rgb(r, g, b) }
 }
 
 /// Sets the drawing color
 pub fn set_draw_color(color: Color) {
+    unsafe {
+        crate::app::open_display();
+    }
     unsafe { Fl_set_color_int(color.bits() as u32) }
 }
 
@@ -250,6 +259,7 @@ pub fn draw_pie(x: i32, y: i32, width: i32, height: i32, a: f64, b: f64) {
 /// Sets the line style
 pub fn set_line_style(style: LineStyle, width: i32) {
     unsafe {
+        crate::app::open_display();
         Fl_line_style(
             style.bits(),
             width,
@@ -420,11 +430,16 @@ pub fn scale_xy(x: f64, y: f64) {
     unsafe { Fl_scale(x, y) }
 }
 
-/// Concatenates scaling transformation onto the current one
+/// Concatenates scaling transformation onto the current one for both x & y
 pub fn scale_x(x: f64) {
     unsafe { Fl_scale2(x) }
 }
 
+/// Concatenates scaling transformation onto the current one for both x & y
+pub fn scale(x: f64) {
+    unsafe { Fl_scale2(x) }
+}
+
 /// Concatenates translation transformation onto the current one
 pub fn translate(x: f64, y: f64) {
     unsafe { Fl_translate(x, y) }
@@ -610,15 +625,19 @@ pub fn rtl_draw(txt: &str, x: i32, y: i32) {
     unsafe { Fl_rtl_draw(txt.as_ptr(), len, x, y) }
 }
 
-/// Draws a frame with text
+/// Draws a series of line segments around the given box.
+/// The string must contain groups of 4 letters which specify one of 24 standard grayscale values,
+/// where 'A' is black and 'X' is white. The order of each set of 4 characters is: top, left, bottom, right.
 pub fn draw_frame(string: &str, x: i32, y: i32, width: i32, height: i32) {
+    assert!(string.len() % 4 == 0);
     let s = CString::safe_new(string);
     unsafe { Fl_frame(s.as_ptr(), x, y, width, height) }
 }
 
-/// Draws a frame with text.
-/// Differs from frame() by the order of the line segments
+/// Draws a series of line segments around the given box
+/// Differs from frame() by the order of the line segments which is bottom, right, top, left.
 pub fn draw_frame2(string: &str, x: i32, y: i32, width: i32, height: i32) {
+    assert!(string.len() % 4 == 0);
     let s = CString::safe_new(string);
     unsafe { Fl_frame2(s.as_ptr(), x, y, width, height) }
 }
@@ -841,3 +860,19 @@ pub fn draw_image(
 pub unsafe fn draw_image2(data: &[u8], x: i32, y: i32, w: i32, h: i32, depth: i32, line_data: i32) {
     Fl_draw_image(data.as_ptr(), x, y, w, h, depth, line_data);
 }
+
+#[cfg(feature = "enable-glwindow")]
+/// Start drawing using OpenGL functions inside a widget's draw routine
+/// # Safety
+/// Requires OpenGL support, Only works with SingleWindow
+pub unsafe fn gl_start() {
+    fltk_sys::window::Fl_gl_start();
+}
+
+#[cfg(feature = "enable-glwindow")]
+/// Finish drawing using OpenGL functions inside a widget's draw routine
+/// # Safety
+/// Requires OpenGL support, Only works with SingleWindow
+pub unsafe fn gl_finish() {
+    fltk_sys::window::Fl_gl_finish();
+}
