@@ -1,9 +1,11 @@
 use crate::app::{font_index, FONTS};
 use crate::prelude::{FltkError, FltkErrorKind};
-use crate::utils::FlString;
-use fltk_sys::fl::Fl_get_rgb_color;
-use std::ffi::{CStr, CString};
-use std::path;
+use crate::utils::{self, FlString};
+use fltk_sys::fl;
+use std::{
+    ffi::{CStr, CString},
+    mem, path,
+};
 
 /// Defines label types
 #[repr(i32)]
@@ -35,6 +37,18 @@ pub enum ColorDepth {
     Rgba8 = 4,
 }
 
+/// Implements some convenience methods for ColorDepth
+impl ColorDepth {
+    /// Create a ColorDepth from an u8 value
+    pub fn from_u8(val: u8) -> Result<ColorDepth, FltkError> {
+        if !(1..=4).contains(&val) {
+            Err(FltkError::Internal(FltkErrorKind::FailedOperation))
+        } else {
+            Ok(unsafe { mem::transmute(val) })
+        }
+    }
+}
+
 /// Defines the frame types which can be set using the `set_frame()` and `set_down_frame()` methods
 #[repr(i32)]
 #[derive(Debug, Copy, Clone, PartialEq)]
@@ -158,30 +172,32 @@ pub enum FrameType {
 impl FrameType {
     /// Alias OFlatFrame as OFlatBox
     pub const OFlatBox: FrameType = FrameType::OFlatFrame;
+    /// Alias GtkRoundDownFrame as GtkRoundDownBox
+    pub const GtkRoundDownBox: FrameType = FrameType::GtkRoundDownFrame;
     /// Gets the Frame type by index
     pub fn by_index(idx: usize) -> FrameType {
         let idx = if idx > 56 { 56 } else { idx };
-        unsafe { std::mem::transmute(idx as i32) }
+        unsafe { mem::transmute(idx as i32) }
     }
 
     /// Get the frame's x offset
     pub fn dx(self) -> i32 {
-        unsafe { fltk_sys::fl::Fl_box_dx(self as i32) }
+        unsafe { fl::Fl_box_dx(self as i32) }
     }
 
     /// Get the frame's y offset
     pub fn dy(self) -> i32 {
-        unsafe { fltk_sys::fl::Fl_box_dy(self as i32) }
+        unsafe { fl::Fl_box_dy(self as i32) }
     }
 
     /// Get the frame's width offset
     pub fn dw(self) -> i32 {
-        unsafe { fltk_sys::fl::Fl_box_dw(self as i32) }
+        unsafe { fl::Fl_box_dw(self as i32) }
     }
 
     /// Get the frame's height offset
     pub fn dh(self) -> i32 {
-        unsafe { fltk_sys::fl::Fl_box_dh(self as i32) }
+        unsafe { fl::Fl_box_dh(self as i32) }
     }
 
     /// Swap frames
@@ -189,7 +205,7 @@ impl FrameType {
         unsafe {
             let new_frame = new_frame as i32;
             let old_frame = old_frame as i32;
-            fltk_sys::fl::Fl_set_box_type(old_frame, new_frame);
+            fl::Fl_set_box_type(old_frame, new_frame);
         }
     }
 }
@@ -286,8 +302,12 @@ impl Font {
     /// Returns a font by index, can be queried via the `app::get_font_names()`
     pub fn by_index(idx: usize) -> Font {
         unsafe {
-            if idx < (*FONTS.lock().unwrap()).len() {
-                std::mem::transmute(idx as i32)
+            if let Some(f) = &FONTS {
+                if idx < (f.lock().unwrap()).len() {
+                    mem::transmute(idx as i32)
+                } else {
+                    Font::Helvetica
+                }
             } else {
                 Font::Helvetica
             }
@@ -313,7 +333,7 @@ impl Font {
     pub fn set_font(old: Font, new: &str) {
         let new = CString::safe_new(new);
         unsafe {
-            fltk_sys::fl::Fl_set_font2(old.bits, new.into_raw() as _);
+            fl::Fl_set_font2(old.bits, new.into_raw() as _);
         }
     }
 
@@ -338,7 +358,7 @@ impl Font {
             }
             if let Some(p) = path.to_str() {
                 let path = CString::safe_new(p);
-                let ptr = fltk_sys::fl::Fl_load_font(path.as_ptr());
+                let ptr = fl::Fl_load_font(path.as_ptr());
                 if ptr.is_null() {
                     Err::<String, FltkError>(FltkError::Internal(FltkErrorKind::FailedOperation))
                 } else {
@@ -350,6 +370,15 @@ impl Font {
             }
         }
     }
+
+    /// Get the font's real name
+    pub fn get_name(&self) -> String {
+        unsafe {
+            CStr::from_ptr(fl::Fl_get_font_name(self.bits as i32))
+                .to_string_lossy()
+                .to_string()
+        }
+    }
 }
 
 bitflags! {
@@ -372,8 +401,12 @@ bitflags! {
         const Inactive = 8;
         /// Selection
         const Selection = 15;
+        /// Free
+        const Free = 16;
         /// Gray0
         const Gray0 = 32;
+        /// GrayRamp
+        const GrayRamp = 32;
         /// Dark3
         const Dark3 = 39;
         /// Dark2
@@ -424,24 +457,84 @@ bitflags! {
 impl Color {
     /// Returns a color from RGB
     pub fn from_rgb(r: u8, g: u8, b: u8) -> Color {
-        unsafe { std::mem::transmute(Fl_get_rgb_color(r, g, b)) }
+        unsafe { mem::transmute(fl::Fl_get_rgb_color(r, g, b)) }
     }
 
     /// Returns a color from hex or decimal
     pub fn from_u32(val: u32) -> Color {
-        let (r, g, b) = crate::utils::hex2rgb(val);
+        let (r, g, b) = utils::hex2rgb(val);
         Color::from_rgb(r, g, b)
     }
 
     /// Returns a color from hex or decimal
     pub fn from_hex(val: u32) -> Color {
-        let (r, g, b) = crate::utils::hex2rgb(val);
+        let (r, g, b) = utils::hex2rgb(val);
         Color::from_rgb(r, g, b)
     }
 
     /// Returns a color by index of RGBI
     pub fn by_index(idx: u8) -> Color {
-        unsafe { std::mem::transmute(idx as u32) }
+        unsafe { mem::transmute(idx as u32) }
+    }
+
+    /// Retuns an inactive form of the color
+    pub fn inactive(&self) -> Color {
+        unsafe { mem::transmute(fl::Fl_inactive(self.bits)) }
+    }
+
+    /// Retuns an darker form of the color
+    pub fn darker(&self) -> Color {
+        unsafe { mem::transmute(fl::Fl_darker(self.bits)) }
+    }
+
+    /// Retuns an lighter form of the color
+    pub fn lighter(&self) -> Color {
+        unsafe { mem::transmute(fl::Fl_lighter(self.bits)) }
+    }
+
+    /// Returns a gray color value from black (i == 0) to white (i == FL_NUM_GRAY - 1)
+    pub fn gray_ramp(val: i32) -> Color {
+        unsafe { mem::transmute(fl::Fl_gray_ramp(val)) }
+    }
+
+    /// Returns a gray color value from black (i == 0) to white (i == FL_NUM_GRAY - 1)
+    pub fn color_average(c1: Color, c2: Color, weight: f32) -> Color {
+        unsafe { mem::transmute(fl::Fl_color_average(c1.bits, c2.bits, weight)) }
+    }
+
+    /// Returns a color that contrasts with the background color.
+    pub fn contrast(fg: Color, bg: Color) -> Color {
+        unsafe { mem::transmute(fl::Fl_contrast(fg.bits, bg.bits)) }
+    }
+
+    /// Returns the color closest to the passed grayscale value
+    pub fn gray_scale(g: u8) -> Color {
+        unsafe { mem::transmute(fl::Fl_rgb_color2(g)) }
+    }
+
+    /// Returns the color closest to the passed rgb value
+    pub fn rgb_color(r: u8, g: u8, b: u8) -> Color {
+        unsafe { mem::transmute(fl::Fl_rgb_color(r, g, b)) }
+    }
+
+    /// Get the RGB value of the color
+    pub fn to_rgb(&self) -> (u8, u8, u8) {
+        unsafe {
+            let val = self.bits;
+            let r = ((val >> 24) & 0xff) as u8;
+            let g = ((val >> 16) & 0xff) as u8;
+            let b = ((val >> 8) & 0xff) as u8;
+            let i = (val & 0xff) as u8;
+            if i == 0 {
+                (r, g, b)
+            } else {
+                let val = fl::Fl_cmap(val);
+                let r = ((val >> 24) & 0xff) as u8;
+                let g = ((val >> 16) & 0xff) as u8;
+                let b = ((val >> 8) & 0xff) as u8;
+                (r, g, b)
+            }
+        }
     }
 }
 
@@ -543,7 +636,7 @@ bitflags! {
         /// Zoom Event
         const ZoomEvent = 27;
         /// Window Resize Event.
-        /// Avoid resizing the widget during a resize event
+        /// Avoid resizing the parent during a resize event
         /// to avoid infinite recursion
         const Resize = 28;
     }
@@ -552,13 +645,13 @@ bitflags! {
 impl Event {
     /// Creates an event from an i32 value
     pub fn from_i32(val: i32) -> Event {
-        unsafe { std::mem::transmute(val) }
+        unsafe { mem::transmute(val) }
     }
 }
 
 impl From<i32> for Event {
     fn from(val: i32) -> Event {
-        unsafe { std::mem::transmute(val) }
+        unsafe { mem::transmute(val) }
     }
 }
 
@@ -691,12 +784,12 @@ bitflags! {
 impl Key {
     /// Gets a Key from an i32
     pub fn from_i32(val: i32) -> Key {
-        unsafe { std::mem::transmute(val) }
+        unsafe { mem::transmute(val) }
     }
 
     /// Gets a Key from a char
     pub fn from_char(val: char) -> Key {
-        unsafe { std::mem::transmute(val) }
+        unsafe { mem::transmute(val) }
     }
 }
 
@@ -766,12 +859,12 @@ impl Shortcut {
 
     /// Get the button number
     pub fn button(button_num: i32) -> Shortcut {
-        unsafe { std::mem::transmute(0x0080_0000 << button_num) }
+        unsafe { mem::transmute(0x0080_0000 << button_num) }
     }
 }
 
 bitflags! {
-    /// Defines the types of triggers for widget callback functions
+    /// Defines the types of triggers for widget callback functions. Equivalent to FL_WHEN
     pub struct CallbackTrigger: i32 {
         /// Never
         const Never = 0;
@@ -899,20 +992,20 @@ bitflags! {
 impl std::ops::BitOr<char> for Shortcut {
     type Output = Shortcut;
     fn bitor(self, other: char) -> Self::Output {
-        unsafe { std::mem::transmute(self.bits as i32 | other as i32) }
+        unsafe { mem::transmute(self.bits as i32 | other as i32) }
     }
 }
 
 impl std::ops::BitOr<Key> for Shortcut {
     type Output = Shortcut;
     fn bitor(self, other: Key) -> Self::Output {
-        unsafe { std::mem::transmute(self.bits as i32 | other.bits() as i32) }
+        unsafe { mem::transmute(self.bits as i32 | other.bits() as i32) }
     }
 }
 
 impl std::ops::BitOr<i32> for Align {
     type Output = Align;
     fn bitor(self, rhs: i32) -> Self::Output {
-        unsafe { std::mem::transmute(self.bits | rhs as i32) }
+        unsafe { mem::transmute(self.bits | rhs as i32) }
     }
 }
