@@ -7,7 +7,6 @@ use fltk_sys::group::*;
 use std::{
     ffi::{CStr, CString},
     mem,
-    ops::{Deref, DerefMut},
     os::raw,
 };
 
@@ -344,22 +343,95 @@ impl Pack {
     }
 }
 
+/// Defines Flex types
+#[repr(i32)]
+#[derive(WidgetType, Debug, Copy, Clone, PartialEq)]
+pub enum FlexType {
+    /// row direction
+    Row = 0,
+    /// column direction
+    Column,
+}
+
+/**
+    a Flexbox widget
+    # Example
+    ```rust,no_run
+    use fltk::{prelude::*, *};
+    fn main() {
+        let a = app::App::default();
+        let mut win = window::Window::default().with_size(400, 300);
+        let mut col = group::Flex::default().size_of_parent();
+        col.set_type(group::FlexType::Column);
+        let expanding = button::Button::default().with_label("Expanding");
+        let mut normal = button::Button::default().with_label("Normal");
+        col.set_size(&mut normal, 30);
+        col.end();
+        win.end();
+        win.show();
+        a.run().unwrap();
+    }
+    ```
+*/
+#[derive(WidgetBase, WidgetExt, GroupExt, Debug)]
+pub struct Flex {
+    inner: *mut Fl_Flex,
+    tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
+    is_derived: bool,
+}
+
+impl Flex {
+    /// Set the size of the widget
+    pub fn set_size<W: WidgetExt>(&mut self, w: &mut W, size: i32) {
+        unsafe { Fl_Flex_set_size(self.inner, w.as_widget_ptr() as _, size) }
+    }
+
+    /// Debug the flex layout
+    pub fn debug(&mut self, flag: bool) {
+        unsafe { Fl_Flex_set_debug(self.inner, flag as _) }
+    }
+
+    /// Set the type to be a column
+    pub fn column(mut self) -> Self {
+        self.set_type(FlexType::Column);
+        self
+    }
+
+    /// Set the type to a row
+    pub fn row(mut self) -> Self {
+        self.set_type(FlexType::Row);
+        self
+    }
+
+    /// Recalculate children's coords and sizes
+    pub fn recalc(&mut self) {
+        self.end();
+    }
+}
+
 /**
     Defines a Vertical Grid (custom widget).
     Requires setting the params manually using the `set_params` method, which takes the rows, columns and spacing.
     ```rust,no_run
     use fltk::{prelude::*, *};
-    let mut grid = group::VGrid::new(0, 0, 400, 300, "");
-    grid.set_params(3, 3, 5);
-    button::Button::default();
-    button::Button::default();
-    button::Button::default();
-    button::Button::default();
-    button::Button::default();
-    button::Button::default();
-    button::Button::default();
-    button::Button::default();
-    grid.end();
+    fn main() {
+        let app = app::App::default();
+        let mut win = window::Window::default().with_size(400, 300);
+        let mut grid = group::VGrid::new(0, 0, 400, 300, "");
+        grid.set_params(3, 3, 5);
+        button::Button::default();
+        button::Button::default();
+        button::Button::default();
+        button::Button::default();
+        button::Button::default();
+        button::Button::default();
+        button::Button::default();
+        button::Button::default();
+        grid.end();
+        win.end();
+        win.show();
+        app.run().unwrap();
+    }
     ```
 */
 #[derive(Debug, Clone)]
@@ -370,7 +442,18 @@ pub struct VGrid {
     current: i32,
 }
 
+impl Default for VGrid {
+    fn default() -> Self {
+        Self::new(0, 0, 0, 0, None)
+    }
+}
+
 impl VGrid {
+    /// Constructs a widget with the size of its parent
+    pub fn default_fill() -> Self {
+        Self::default().size_of_parent()
+    }
+
     /// Creates a new vertical grid
     pub fn new<T: Into<Option<&'static str>>>(x: i32, y: i32, w: i32, h: i32, label: T) -> VGrid {
         let vpack = Pack::new(x, y, w, h, label);
@@ -400,7 +483,7 @@ impl VGrid {
 
     /// Adds widgets to the grid
     pub fn add<W: WidgetExt>(&mut self, w: &W) {
-        debug_assert!(self.current + 1 <= self.rows * self.cols);
+        debug_assert!(self.current < self.rows * self.cols);
         let rem = (self.current - 1) / self.cols;
         if rem < self.rows {
             let hpack = self.vpack.child(rem as i32).unwrap();
@@ -440,36 +523,31 @@ impl VGrid {
     }
 }
 
-impl Deref for VGrid {
-    type Target = Pack;
-
-    fn deref(&self) -> &Self::Target {
-        &self.vpack
-    }
-}
-
-impl DerefMut for VGrid {
-    fn deref_mut(&mut self) -> &mut Self::Target {
-        &mut self.vpack
-    }
-}
+crate::widget_extends!(VGrid, Pack, vpack);
 
 /**
     Defines a Horizontal Grid (custom widget).
     Requires setting the params manually using the `set_params` method, which takes the rows, columns and spacing.
     ```rust,no_run
     use fltk::{prelude::*, *};
-    let mut grid = group::HGrid::new(0, 0, 400, 300, "");
-    grid.set_params(3, 3, 5);
-    button::Button::default();
-    button::Button::default();
-    button::Button::default();
-    button::Button::default();
-    button::Button::default();
-    button::Button::default();
-    button::Button::default();
-    button::Button::default();
-    grid.end();
+    fn main() {
+        let app = app::App::default();
+        let mut win = window::Window::default().with_size(400, 300);
+        let mut grid = group::HGrid::new(0, 0, 400, 300, "");
+        grid.set_params(3, 3, 5);
+        button::Button::default();
+        button::Button::default();
+        button::Button::default();
+        button::Button::default();
+        button::Button::default();
+        button::Button::default();
+        button::Button::default();
+        button::Button::default();
+        grid.end();
+        win.end();
+        win.show();
+        app.run().unwrap();
+    }
     ```
 */
 #[derive(Debug, Clone)]
@@ -480,7 +558,18 @@ pub struct HGrid {
     current: i32,
 }
 
+impl Default for HGrid {
+    fn default() -> Self {
+        Self::new(0, 0, 0, 0, None)
+    }
+}
+
 impl HGrid {
+    /// Constructs a widget with the size of its parent
+    pub fn default_fill() -> Self {
+        Self::default().size_of_parent()
+    }
+
     /// Creates a new horizontal grid
     pub fn new<T: Into<Option<&'static str>>>(x: i32, y: i32, w: i32, h: i32, label: T) -> HGrid {
         let mut hpack = Pack::new(x, y, w, h, label);
@@ -510,7 +599,7 @@ impl HGrid {
 
     /// Adds widgets to the grid
     pub fn add<W: WidgetExt>(&mut self, w: &W) {
-        debug_assert!(self.current + 1 <= self.rows * self.cols);
+        debug_assert!(self.current < self.rows * self.cols);
         let rem = (self.current - 1) / self.rows;
         if rem < self.cols {
             let vpack = self.hpack.child(rem as i32).unwrap();
@@ -550,109 +639,82 @@ impl HGrid {
     }
 }
 
-impl Deref for HGrid {
-    type Target = Pack;
-
-    fn deref(&self) -> &Self::Target {
-        &self.hpack
-    }
-}
-
-impl DerefMut for HGrid {
-    fn deref_mut(&mut self) -> &mut Self::Target {
-        &mut self.hpack
-    }
-}
+crate::widget_extends!(HGrid, Pack, hpack);
 
 /// A wrapper around a vertical pack, with `auto_layout`ing using the add method
 #[derive(Debug, Clone)]
 pub struct Column {
-    p: Pack,
+    p: Flex,
+}
+
+impl Default for Column {
+    fn default() -> Self {
+        Self::new(0, 0, 0, 0, None)
+    }
 }
 
 impl Column {
-    /// Create a new column
-    pub fn new<T: Into<Option<&'static str>>>(x: i32, y: i32, w: i32, h: i32, label: T) -> Column {
-        let p = Pack::new(x, y, w, h, label);
-        Column { p }
+    /// Constructs a widget with the size of its parent
+    pub fn default_fill() -> Self {
+        Self::default().size_of_parent()
     }
 
-    /// Default init a column filling the parent
-    pub fn default() -> Self {
-        let mut p = Pack::default().size_of_parent().center_of_parent();
-        p.set_type(PackType::Vertical);
+    /// Create a new column
+    pub fn new<T: Into<Option<&'static str>>>(
+        x: i32,
+        y: i32,
+        width: i32,
+        height: i32,
+        label: T,
+    ) -> Column {
+        let mut p = Flex::new(x, y, width, height, label);
+        p.set_type(FlexType::Column);
         Column { p }
     }
 
     /// Add a widget to the column with automatic layouting
     pub fn add<W: WidgetExt>(&mut self, w: &W) {
         self.p.add(w);
-        self.p.auto_layout();
-    }
-
-    /// End the column
-    pub fn end(&mut self) {
-        self.p.auto_layout();
     }
 }
 
-impl Deref for Column {
-    type Target = Pack;
-
-    fn deref(&self) -> &Self::Target {
-        &self.p
-    }
-}
-
-impl DerefMut for Column {
-    fn deref_mut(&mut self) -> &mut Self::Target {
-        &mut self.p
-    }
-}
+crate::widget_extends!(Column, Flex, p);
 
 /// A wrapper around a Horizontal pack, with `auto_layout`ing using the add method
 #[derive(Debug, Clone)]
 pub struct Row {
-    p: Pack,
+    p: Flex,
+}
+
+impl Default for Row {
+    fn default() -> Self {
+        Self::new(0, 0, 0, 0, None)
+    }
 }
 
 impl Row {
-    /// Create a new row
-    pub fn new<T: Into<Option<&'static str>>>(x: i32, y: i32, w: i32, h: i32, label: T) -> Row {
-        let mut p = Pack::new(x, y, w, h, label);
-        p.set_type(PackType::Horizontal);
-        Row { p }
+    /// Constructs a widget with the size of its parent
+    pub fn default_fill() -> Self {
+        Self::default().size_of_parent().center_of_parent()
     }
 
-    /// Default init a row filling the parent
-    pub fn default() -> Self {
-        let mut p = Pack::default().size_of_parent().center_of_parent();
-        p.set_type(PackType::Horizontal);
+    /// Create a new row
+    pub fn new<T: Into<Option<&'static str>>>(
+        x: i32,
+        y: i32,
+        width: i32,
+        height: i32,
+        label: T,
+    ) -> Row {
+        let mut p = Flex::new(x, y, width, height, label);
+        p.set_type(FlexType::Row);
         Row { p }
     }
 
     /// Add a widget to the row with automatic layouting
     pub fn add<W: WidgetExt>(&mut self, w: &W) {
         self.p.add(w);
-        self.p.auto_layout();
-    }
-
-    /// End the row
-    pub fn end(&mut self) {
-        self.p.auto_layout();
     }
 }
 
-impl Deref for Row {
-    type Target = Pack;
-
-    fn deref(&self) -> &Self::Target {
-        &self.p
-    }
-}
-
-impl DerefMut for Row {
-    fn deref_mut(&mut self) -> &mut Self::Target {
-        &mut self.p
-    }
-}
+crate::widget_extends!(Row, Flex, p);
