@@ -2,13 +2,23 @@ use crate::enums::{Align, CallbackTrigger, Color, Damage, Event, Font, FrameType
 use crate::image::Image;
 use crate::prelude::*;
 use crate::utils::FlString;
-use fltk_sys::output::*;
+use fltk_sys::input::*;
 use std::{
     ffi::{CStr, CString},
     mem,
     os::raw,
 };
 
+/// Sets the input widget's type
+#[repr(i32)]
+#[derive(WidgetType, Debug, Copy, Clone, PartialEq)]
+pub enum OutputType {
+    /// Normal input
+    Normal = 8,
+    /// Multiline input
+    Multiline = 12,
+}
+
 /// Creates an output widget
 #[derive(WidgetBase, WidgetExt, InputExt, Debug)]
 pub struct Output {
