@@ -105,6 +105,58 @@ pub trait WidgetType {
 
 /// Defines the methods implemented by all widgets
 pub unsafe trait WidgetExt {
+    /// Initialize to a position x, y
+    fn with_pos(self, x: i32, y: i32) -> Self
+    where
+        Self: Sized;
+    /// Initialize to size width, height
+    fn with_size(self, width: i32, height: i32) -> Self
+    where
+        Self: Sized;
+    /// Initialize with a label
+    fn with_label(self, title: &str) -> Self
+    where
+        Self: Sized;
+    /// Initialize with alignment
+    fn with_align(self, align: crate::enums::Align) -> Self
+    where
+        Self: Sized;
+    /// Initialize with type
+    fn with_type<T: WidgetType>(self, typ: T) -> Self
+    where
+        Self: Sized;
+    /// Initialize at bottom of another widget
+    fn below_of<W: WidgetExt>(self, wid: &W, padding: i32) -> Self
+    where
+        Self: Sized;
+    /// Initialize above of another widget
+    fn above_of<W: WidgetExt>(self, wid: &W, padding: i32) -> Self
+    where
+        Self: Sized;
+    /// Initialize right of another widget
+    fn right_of<W: WidgetExt>(self, wid: &W, padding: i32) -> Self
+    where
+        Self: Sized;
+    /// Initialize left of another widget
+    fn left_of<W: WidgetExt>(self, wid: &W, padding: i32) -> Self
+    where
+        Self: Sized;
+    /// Initialize center of another widget
+    fn center_of<W: WidgetExt>(self, w: &W) -> Self
+    where
+        Self: Sized;
+    /// Initialize center of parent
+    fn center_of_parent(self) -> Self
+    where
+        Self: Sized;
+    /// Initialize to the size of another widget
+    fn size_of<W: WidgetExt>(self, w: &W) -> Self
+    where
+        Self: Sized;
+    /// Initialize to the size of the parent
+    fn size_of_parent(self) -> Self
+    where
+        Self: Sized;
     /// Set to position x, y
     fn set_pos(&mut self, x: i32, y: i32);
     /// Set to dimensions width and height
@@ -139,54 +191,6 @@ pub unsafe trait WidgetExt {
     /// # Safety
     /// Can return multiple mutable pointers to the same widget
     unsafe fn as_widget_ptr(&self) -> *mut fltk_sys::widget::Fl_Widget;
-    /// Initialize to position x, y, (should only be called on initialization)
-    fn with_pos(self, x: i32, y: i32) -> Self
-    where
-        Self: Sized;
-    /// Initialilze to dimensions width and height, (should only be called on initialization)
-    fn with_size(self, width: i32, height: i32) -> Self
-    where
-        Self: Sized;
-    /// Initialize with label/title, (should only be called on initialization)
-    fn with_label(self, title: &str) -> Self
-    where
-        Self: Sized;
-    /// Sets the initial alignment of the widget, (should only be called on initialization)
-    fn with_align(self, align: Align) -> Self
-    where
-        Self: Sized;
-    /// Positions the widget below w, the size of w should be known
-    fn below_of<W: WidgetExt>(self, w: &W, padding: i32) -> Self
-    where
-        Self: Sized;
-    /// Positions the widget above w, the size of w should be known
-    fn above_of<W: WidgetExt>(self, w: &W, padding: i32) -> Self
-    where
-        Self: Sized;
-    /// Positions the widget to the right of w, the size of w should be known
-    fn right_of<W: WidgetExt>(self, w: &W, padding: i32) -> Self
-    where
-        Self: Sized;
-    /// Positions the widget to the left of w, the size of w should be known
-    fn left_of<W: WidgetExt>(self, w: &W, padding: i32) -> Self
-    where
-        Self: Sized;
-    /// Positions the widget to the center of w, the size of w should be known
-    fn center_of<W: WidgetExt>(self, w: &W) -> Self
-    where
-        Self: Sized;
-    /// Positions the widget to the center of its parent
-    fn center_of_parent(self) -> Self
-    where
-        Self: Sized;
-    /// Takes the size of w, the size of w should be known
-    fn size_of<W: WidgetExt>(self, w: &W) -> Self
-    where
-        Self: Sized;
-    /// Takes the size of the parent group or window
-    fn size_of_parent(self) -> Self
-    where
-        Self: Sized;
     /// Checks whether the self widget is inside another widget
     fn inside<W: WidgetExt>(&self, wid: &W) -> bool
     where
@@ -304,6 +308,8 @@ pub unsafe trait WidgetExt {
     fn visible_focus(&mut self, v: bool);
     /// Return whether the widget has visible focus
     fn has_visible_focus(&mut self) -> bool;
+    /// Return whether the widget has focus
+    fn has_focus(&mut self) -> bool;
     /// Check if a widget was deleted
     fn was_deleted(&self) -> bool;
     /// Return whether the widget was damaged
@@ -316,9 +322,9 @@ pub unsafe trait WidgetExt {
     fn set_damage_type(&mut self, mask: Damage);
     /// Clear the damaged flag
     fn clear_damage(&mut self);
-    /// Sets the default callback trigger for a widget
+    /// Sets the default callback trigger for a widget, equivalent to `when()`
     fn set_trigger(&mut self, trigger: CallbackTrigger);
-    /// Return the callback trigger
+    /// Return the callback trigger, equivalent to `when()`
     fn trigger(&self) -> CallbackTrigger;
     /// Return the widget as a window if it's a window
     fn as_window(&self) -> Option<Box<dyn WindowExt>>;
@@ -347,21 +353,31 @@ pub unsafe trait WidgetExt {
     fn active(&self) -> bool;
     /// Returns whether a widget or any of its parents are active (recursively)
     fn active_r(&self) -> bool;
+    #[doc(hidden)]
     /**
-        Return the default callback function, this allows storing then running within the overriden callback
-        ```rust
+        Return the default callback function, this allows storing then running within the overriden callback.
+        Works only for FLTK types (with no callback defined in the Rust side)
+        ```rust,no_run
             use fltk::{prelude::*, *};
-            let scroll = group::Scroll::default();
-            let mut scrollbar = scroll.scrollbar();
-            scrollbar.set_callback({
-            let mut cb = scrollbar.callback();
-                move |_| {
-                    println!("print something, and also run the default callback");
-                    if let Some(cb) = cb.as_mut() {
-                        (*cb)();
+            fn main() {
+                let a = app::App::default();
+                let mut win = window::Window::default().with_size(400, 300);
+                let scroll = group::Scroll::default().size_of_parent();
+                let _btn = button::Button::new(160, 500, 80, 40, "click");
+                let mut scrollbar = scroll.scrollbar();
+                scrollbar.set_callback({
+                    let mut cb = scrollbar.callback();
+                    move |_| {
+                        println!("print something, and also run the default callback");
+                        if let Some(cb) = cb.as_mut() {
+                            (*cb)();
+                        }
                     }
-                }
-            });
+                });
+                win.end();
+                win.show();
+                a.run().unwrap();
+            }
         ```
     */
     fn callback(&self) -> Option<Box<dyn FnMut()>>;
@@ -389,6 +405,8 @@ pub unsafe trait WidgetBase: WidgetExt {
         height: i32,
         title: T,
     ) -> Self;
+    /// Constructs a widget with the size of its parent
+    fn default_fill() -> Self;
     /// Deletes widgets and their children.
     fn delete(wid: Self)
     where
@@ -417,6 +435,9 @@ pub unsafe trait WidgetBase: WidgetExt {
     /// # Safety
     /// Can return multiple mutable references to the `handle_data`
     unsafe fn handle_data(&mut self) -> Option<Box<dyn FnMut(Event) -> bool>>;
+    /// Perform a callback on resize.
+    /// Avoid resizing the parent or the same widget to avoid infinite recursion
+    fn resize_callback<F: FnMut(&mut Self, i32, i32, i32, i32) + 'static>(&mut self, cb: F);
 }
 
 /// Defines the methods implemented by all button widgets
@@ -701,7 +722,8 @@ pub unsafe trait MenuExt: WidgetExt {
         shortcut: Shortcut,
         flag: crate::menu::MenuFlag,
         cb: F,
-    ) -> i32 where
+    ) -> i32
+    where
         Self: Sized;
     /// Inserts a menu item at an index along with its callback.
     /// The characters "&", "/", "\\", and "\_" (underscore) are treated as special characters in the label string. The "&" character specifies that the following character is an accelerator and will be underlined.
@@ -714,7 +736,8 @@ pub unsafe trait MenuExt: WidgetExt {
         shortcut: Shortcut,
         flag: crate::menu::MenuFlag,
         cb: F,
-    ) -> i32 where
+    ) -> i32
+    where
         Self: Sized;
     /// Add a menu item along with an emit (sender and message).
     /// The characters "&", "/", "\\", and "\_" (underscore) are treated as special characters in the label string. The "&" character specifies that the following character is an accelerator and will be underlined.
@@ -726,7 +749,8 @@ pub unsafe trait MenuExt: WidgetExt {
         flag: crate::menu::MenuFlag,
         sender: crate::app::Sender<T>,
         msg: T,
-    ) -> i32 where
+    ) -> i32
+    where
         Self: Sized;
     /// Inserts a menu item along with an emit (sender and message).
     /// The characters "&", "/", "\\", and "\_" (underscore) are treated as special characters in the label string. The "&" character specifies that the following character is an accelerator and will be underlined.
@@ -739,7 +763,8 @@ pub unsafe trait MenuExt: WidgetExt {
         flag: crate::menu::MenuFlag,
         sender: crate::app::Sender<T>,
         msg: T,
-    ) -> i32 where
+    ) -> i32
+    where
         Self: Sized;
     /// Remove a menu item by index
     fn remove(&mut self, idx: i32);
@@ -787,6 +812,12 @@ pub unsafe trait MenuExt: WidgetExt {
     fn down_frame(&self) -> FrameType;
     /// Make a menu globally accessible from any window
     fn global(&mut self);
+    /// Get the menu element
+    fn menu(&self) -> Option<crate::menu::MenuItem>;
+    /// Set the menu element
+    /// # Safety
+    /// The MenuItem must be in a format recognized by FLTK (Null termination after submenus)
+    unsafe fn set_menu(&mut self, item: crate::menu::MenuItem);
 }
 
 /// Defines the methods implemented by all valuator widgets
@@ -882,6 +913,8 @@ pub unsafe trait DisplayExt: WidgetExt {
         style_buffer: B,
         entries: Vec<crate::text::StyleTableEntry>,
     );
+    /// Unset the style of the text widget
+    fn unset_highlight_data(&mut self, style_buffer: crate::text::TextBuffer);
     /// Sets the cursor style
     fn set_cursor_style(&mut self, style: crate::text::Cursor);
     /// Sets the cursor color
@@ -1212,6 +1245,8 @@ pub unsafe trait ImageExt {
         Self: Sized;
     /// Draws the image at the presupplied coordinates and size
     fn draw(&mut self, x: i32, y: i32, width: i32, height: i32);
+    /// Draws the image at the presupplied coordinates and size and offset cx, cy
+    fn draw_ext(&mut self, x: i32, y: i32, width: i32, height: i32, cx: i32, cy: i32);
     /// Return the width of the image
     fn width(&self) -> i32;
     /// Return the height of the image
@@ -1240,7 +1275,7 @@ pub unsafe trait ImageExt {
     fn to_rgb(&self) -> Result<crate::image::RgbImage, FltkError>;
     /// Scales the image
     fn scale(&mut self, width: i32, height: i32, proportional: bool, can_expand: bool);
-    /// Return the count of an image
+    /// Return the count of pointers in an image (Pixmaps have more than 1, bitmaps have 0, Rgb based images have 1)
     fn count(&self) -> i32;
     /// Gets the image's data width
     fn data_w(&self) -> i32;
@@ -1287,3 +1322,206 @@ pub trait SurfaceDevice {
     /// Pop the current surface
     fn pop_current();
 }
+
+/// Defines a set of convenience functions for constructing and anchoring custom widgets.
+/// Usage: fltk::widget_extends!(CustomWidget, BaseWidget, member);
+/// It basically implements Deref and DerefMut on the custom widget, and adds the aforementioned methods.
+#[macro_export]
+macro_rules! widget_extends {
+    ($widget:ty, $base:ty, $member:tt) => {
+        impl $widget {
+            /// Initialize to position x, y
+            pub fn with_pos(mut self, x: i32, y: i32) -> Self {
+                let w = self.w();
+                let h = self.h();
+                self.resize(x, y, w, h);
+                self
+            }
+
+            /// Initialize to size width, height
+            pub fn with_size(mut self, width: i32, height: i32) -> Self {
+                let x = self.x();
+                let y = self.y();
+                let w = self.width();
+                let h = self.height();
+                if w == 0 || h == 0 {
+                    self.widget_resize(x, y, width, height);
+                } else {
+                    self.resize(x, y, width, height);
+                }
+                self
+            }
+
+            /// Initialize with a label
+            pub fn with_label(mut self, title: &str) -> Self {
+                self.set_label(title);
+                self
+            }
+
+            /// Initialize with alignment
+            pub fn with_align(mut self, align: $crate::enums::Align) -> Self {
+                self.set_align(align);
+                self
+            }
+
+            /// Initialize with type
+            pub fn with_type<T: $crate::prelude::WidgetType>(mut self, typ: T) -> Self {
+                assert!(!self.was_deleted());
+                self.set_type(typ);
+                self
+            }
+
+            /// Initialize at bottom of another widget
+            pub fn below_of<W: $crate::prelude::WidgetExt>(
+                mut self,
+                wid: &W,
+                padding: i32,
+            ) -> Self {
+                assert!(!wid.was_deleted());
+                assert!(!self.was_deleted());
+                let w = self.w();
+                let h = self.h();
+                debug_assert!(
+                    w != 0 && h != 0,
+                    "below_of requires the size of the widget to be known!"
+                );
+                self.resize(wid.x(), wid.y() + wid.h() + padding, w, h);
+                self
+            }
+
+            /// Initialize above of another widget
+            pub fn above_of<W: $crate::prelude::WidgetExt>(
+                mut self,
+                wid: &W,
+                padding: i32,
+            ) -> Self {
+                assert!(!wid.was_deleted());
+                assert!(!self.was_deleted());
+                let w = self.w();
+                let h = self.h();
+                debug_assert!(
+                    w != 0 && h != 0,
+                    "above_of requires the size of the widget to be known!"
+                );
+                self.resize(wid.x(), wid.y() - padding - h, w, h);
+                self
+            }
+
+            /// Initialize right of another widget
+            pub fn right_of<W: $crate::prelude::WidgetExt>(
+                mut self,
+                wid: &W,
+                padding: i32,
+            ) -> Self {
+                assert!(!wid.was_deleted());
+                assert!(!self.was_deleted());
+                let w = self.w();
+                let h = self.h();
+                debug_assert!(
+                    w != 0 && h != 0,
+                    "right_of requires the size of the widget to be known!"
+                );
+                self.resize(wid.x() + wid.width() + padding, wid.y(), w, h);
+                self
+            }
+
+            /// Initialize left of another widget
+            pub fn left_of<W: $crate::prelude::WidgetExt>(mut self, wid: &W, padding: i32) -> Self {
+                assert!(!wid.was_deleted());
+                assert!(!self.was_deleted());
+                let w = self.w();
+                let h = self.h();
+                debug_assert!(
+                    w != 0 && h != 0,
+                    "left_of requires the size of the widget to be known!"
+                );
+                self.resize(wid.x() - w - padding, wid.y(), w, h);
+                self
+            }
+
+            /// Initialize center of another widget
+            pub fn center_of<W: $crate::prelude::WidgetExt>(mut self, w: &W) -> Self {
+                assert!(!w.was_deleted());
+                assert!(!self.was_deleted());
+                debug_assert!(
+                    w.width() != 0 && w.height() != 0,
+                    "center_of requires the size of the widget to be known!"
+                );
+                let sw = self.width() as f64;
+                let sh = self.height() as f64;
+                let ww = w.width() as f64;
+                let wh = w.height() as f64;
+                let sx = (ww - sw) / 2.0;
+                let sy = (wh - sh) / 2.0;
+                let wx = if w.as_window().is_some() { 0 } else { w.x() };
+                let wy = if w.as_window().is_some() { 0 } else { w.y() };
+                self.resize(sx as i32 + wx, sy as i32 + wy, sw as i32, sh as i32);
+                self.redraw();
+                self
+            }
+
+            /// Initialize center of parent
+            pub fn center_of_parent(mut self) -> Self {
+                assert!(!self.was_deleted());
+                if let Some(w) = self.parent() {
+                    debug_assert!(
+                        w.width() != 0 && w.height() != 0,
+                        "center_of requires the size of the widget to be known!"
+                    );
+                    let sw = self.width() as f64;
+                    let sh = self.height() as f64;
+                    let ww = w.width() as f64;
+                    let wh = w.height() as f64;
+                    let sx = (ww - sw) / 2.0;
+                    let sy = (wh - sh) / 2.0;
+                    let wx = if w.as_window().is_some() { 0 } else { w.x() };
+                    let wy = if w.as_window().is_some() { 0 } else { w.y() };
+                    self.resize(sx as i32 + wx, sy as i32 + wy, sw as i32, sh as i32);
+                    self.redraw();
+                }
+                self
+            }
+
+            /// Initialize to the size of another widget
+            pub fn size_of<W: $crate::prelude::WidgetExt>(mut self, w: &W) -> Self {
+                assert!(!w.was_deleted());
+                assert!(!self.was_deleted());
+                debug_assert!(
+                    w.width() != 0 && w.height() != 0,
+                    "size_of requires the size of the widget to be known!"
+                );
+                let x = self.x();
+                let y = self.y();
+                self.resize(x, y, w.width(), w.height());
+                self
+            }
+
+            /// Initialize to the size of the parent
+            pub fn size_of_parent(mut self) -> Self {
+                assert!(!self.was_deleted());
+                if let Some(parent) = self.parent() {
+                    let w = parent.width();
+                    let h = parent.height();
+                    let x = self.x();
+                    let y = self.y();
+                    self.resize(x, y, w, h);
+                }
+                self
+            }
+        }
+
+        impl std::ops::Deref for $widget {
+            type Target = $base;
+
+            fn deref(&self) -> &Self::Target {
+                &self.$member
+            }
+        }
+
+        impl std::ops::DerefMut for $widget {
+            fn deref_mut(&mut self) -> &mut Self::Target {
+                &mut self.$member
+            }
+        }
+    };
+}
