@@ -285,13 +285,13 @@ impl TextBuffer {
     pub fn selection_position(&mut self) -> Option<(i32, i32)> {
         assert!(!self.inner.is_null());
         unsafe {
-            let start: *mut raw::c_int = std::ptr::null_mut();
-            let end: *mut raw::c_int = std::ptr::null_mut();
-            let ret = Fl_Text_Buffer_selection_position(self.inner, start, end);
+            let mut start = 0;
+            let mut end = 0;
+            let ret = Fl_Text_Buffer_selection_position(self.inner, &mut start as _, &mut end as _);
             if ret == 0 {
                 None
             } else {
-                let x = (*start as i32, *end as i32);
+                let x = (start as i32, end as i32);
                 Some(x)
             }
         }
@@ -344,13 +344,13 @@ impl TextBuffer {
     pub fn highlight_position(&mut self) -> Option<(i32, i32)> {
         assert!(!self.inner.is_null());
         unsafe {
-            let start: *mut raw::c_int = std::ptr::null_mut();
-            let end: *mut raw::c_int = std::ptr::null_mut();
-            let ret = Fl_Text_Buffer_highlight_position(self.inner, start, end);
+            let mut start = 0;
+            let mut end = 0;
+            let ret = Fl_Text_Buffer_highlight_position(self.inner, &mut start as _, &mut end as _);
             if ret == 0 {
                 None
             } else {
-                let x = (*start as i32, *end as i32);
+                let x = (start as i32, end as i32);
                 Some(x)
             }
         }
@@ -489,6 +489,92 @@ impl TextBuffer {
             Fl_Text_Buffer_remove_modify_callback(self.inner, callback, data);
         }
     }
+
+    /// Forward search for a string
+    pub fn search_forward(
+        &self,
+        start_pos: i32,
+        search_string: &str,
+        match_case: bool,
+    ) -> Option<i32> {
+        unsafe {
+            let search_string = CString::safe_new(search_string);
+            let mut found_pos = 0;
+            let ret = Fl_Text_Buffer_search_forward(
+                self.inner,
+                start_pos,
+                search_string.as_ptr() as _,
+                &mut found_pos as _,
+                match_case as _,
+            );
+            if ret == 0 {
+                None
+            } else {
+                Some(found_pos)
+            }
+        }
+    }
+
+    /// Backward search for a string
+    pub fn search_backward(
+        &self,
+        start_pos: i32,
+        search_string: &str,
+        match_case: bool,
+    ) -> Option<i32> {
+        unsafe {
+            let search_string = CString::safe_new(search_string);
+            let mut found_pos = 0;
+            let ret = Fl_Text_Buffer_search_backward(
+                self.inner,
+                start_pos,
+                search_string.as_ptr() as _,
+                &mut found_pos as _,
+                match_case as _,
+            );
+            if ret == 0 {
+                None
+            } else {
+                Some(found_pos)
+            }
+        }
+    }
+
+    /// Forward search for a char
+    pub fn find_char_forward(&self, start_pos: i32, search_char: char) -> Option<i32> {
+        unsafe {
+            let mut found_pos = 0;
+            let ret = Fl_Text_Buffer_findchar_forward(
+                self.inner,
+                start_pos,
+                search_char as _,
+                &mut found_pos as _,
+            );
+            if ret == 0 {
+                None
+            } else {
+                Some(found_pos)
+            }
+        }
+    }
+
+    /// Backward search for a char
+    pub fn find_char_backward(&self, start_pos: i32, search_char: char) -> Option<i32> {
+        unsafe {
+            let mut found_pos = 0;
+            let ret = Fl_Text_Buffer_findchar_backward(
+                self.inner,
+                start_pos,
+                search_char as _,
+                &mut found_pos as _,
+            );
+            if ret == 0 {
+                None
+            } else {
+                Some(found_pos)
+            }
+        }
+    }
 }
 
 unsafe impl Sync for TextBuffer {}
